#ifndef AUTONOME_H
#define AUTONOME_H

struct Acteur
{
   virtual void agir() = 0;
   virtual ~Acteur() throw() = 0;
};

class Rythmique
{
public:
   typedef long delai_type;
private:
   delai_type delai_;
public:
   Rythmique(const delai_type delai = 0) throw()
      : delai_(delai)
   {
   }
   delai_type delai() const volatile throw()
      { return delai_; }
   void delai(const delai_type d) volatile throw()
      { delai_ = d; }
   void suspendre() const volatile;
};

#include "Evenement.h"

class Stoppable
{
   bool arret_demande_, arret_complete_;
   Evenement evenement_;
public:
   Stoppable()
      : arret_demande_(false), arret_complete_(false),
        evenement_(Evenement::ManualReset())
   {
   }
   bool arret_complete() const volatile throw()
      { return arret_complete_; }
   bool arret_demande() const volatile throw()
      { return arret_demande_; }
   void demande_arret() volatile
      { arret_demande_ = true; }
   void arret_complet() volatile
   {
      arret_complete_ = true;
      evenement_.provoquer();
   }
   bool attendre_arret(const int delai = Evenement::INFINI) const volatile
      { return evenement_.attendre(delai); }
   ~Stoppable() throw()
      { }
};

class ThreadRep;

class Autonome
   : public Acteur, public Rythmique, public Stoppable
{
   ThreadRep *rep_;
protected:
   Autonome(const delai_type = 0);
   bool demarrer();
   void arreter() throw();
   virtual void debut_impl()
      { }
   virtual void fin_impl()
      { }
public:
   virtual ~Autonome() throw ()
      { arreter(); }
   void debut()
      { debut_impl(); }
   void fin()
      { fin_impl(); }
};

#endif
