#ifndef TAMPON_SECURISE_H
#define TAMPON_SECURISE_H

#include "Mutex.h"
#include <algorithm>

template <class C>
   class TamponSecurise
   {
   public:
      typedef typename
         C::size_type size_type;
      C contenu_;
      Mutex mutex_;
   public:
      bool empty() const
         { return contenu_.empty(); }
      bool empty() const volatile
      {
         Autoverrou av(mutex_);
         return const_cast<const TamponSecurise<C>*>(this)->empty();
      }
      void ajouter(const C &c)
         { ajouter(c.begin(), c.end()); }
      template <class Itt>
         void ajouter(Itt debut, Itt fin)
            { contenu_.append(debut, fin); }
      void ajouter(const C &c) volatile
      {
         Autoverrou av(mutex_);
         const_cast<TamponSecurise<C>*>(this)->ajouter(c);
      }
      template <class Itt>
         void ajouter(Itt debut, Itt fin) volatile
         {
            Autoverrou av(mutex_);
            const_cast<TamponSecurise<C>*>(this)->ajouter(debut, fin);
         }
      C extraire()
      {
         C temp = contenu_;
         contenu_.clear();
         return temp;
      }
      C extraire(const size_type n)
      {
         if (n >= contenu_.size()) 
            return extraire();
         typedef typename
            C::iterator iterator;
         iterator itt = contenu_.begin();
         std::advance(itt, n);
         C temp(contenu_.begin(), itt);
         contenu_.erase(contenu_.begin(), itt);
         return temp;
      }
      C extraire() volatile
      {
         Autoverrou av(mutex_);
         return const_cast<TamponSecurise<C>*>(this)->extraire();
      }
      C extraire(const size_type n) volatile
      {
         Autoverrou av(mutex_);
         return const_cast<TamponSecurise<C>*>(this)->extraire(n);
      }
   };

#endif

