#include "ISocket.h"
#include "SocketServeur.h"
#include "TamponSecurise.h"
#include "Autonome.h"
#include <iostream>
using std::cerr;
using std::cout;
using std::cin;
using std::endl;
#include <algorithm>
using std::copy;
#include <string>
using std::string;
#include <iterator>
using std::ostream_iterator;
#include <utility>
using std::pair;

template <class T, class C>
   class RecepteurAsynchrone
      : public Autonome
   {
      T &entree_;
      volatile TamponSecurise<C> &recu_;
      RecepteurAsynchrone(T &entree, volatile TamponSecurise<C> &t)
         : entree_(entree), recu_(t)
      {
      }
   public:
      typedef typename
         C::size_type size_type;
      static RecepteurAsynchrone *creer(T &entree, volatile TamponSecurise<C> &t)
      {
         RecepteurAsynchrone *p = new RecepteurAsynchrone(entree, t);
         p->demarrer();
         return p;
      }
      void agir()
      {
         try
         {
            const size_type TAILLE = 1024*16;
            char tampon[TAILLE];
            size_type n = entree_->recevoir<char>(tampon, tampon + TAILLE);
            if (n)
               recu_.ajouter(C(tampon, tampon + n));
         }
         catch (pair<SocketsFlux::ISocket::FermetureHomologue,
                     SocketsFlux::ISocket::erreur_type> &p)
         {
            if (p.first.brutale())
               cerr << "Fermeture brutale de l'homologue" << endl;
            else
               cerr << "Fermeture propre de l'homologue" << endl;
            demande_arret();
         }
         if (!recu_.empty())
         {
            C s = recu_.extraire();
            copy(s.begin(), s.end(), ostream_iterator<char>(cout));
         }
      }
   };

#include <memory>
using std::auto_ptr;

int main()
{
   const SocketsFlux::Port PORT_SERVEUR(15234);
   SocketsFlux::SocketServeur serveur(PORT_SERVEUR);
   TamponSecurise <std::string> tampon;

   SocketsFlux::ISocket* conversation = serveur.accepter();
   auto_ptr<RecepteurAsynchrone <SocketsFlux::ISocket *, string> > recepteur
      (RecepteurAsynchrone<SocketsFlux::ISocket*, string>::creer(conversation, tampon));
   auto_ptr<SocketsFlux::ISocket> wrap_conv(conversation);
   char c;
   cin >> c;
}
